from __future__ import annotations

from typing import TYPE_CHECKING, List, Dict, Union

if TYPE_CHECKING:
    from alaric.abc import ComparisonT, LogicalT, Buildable


class NOT:
    """Invert the effect of a query expression.

    This back-links to `here <https://www.mongodb.com/docs/manual/reference/operator/query/not/#mongodb-query-op.-not>`_


    Lets find all items where my ``gamer_tag`` does NOT match ``Skelmis``

    .. code-block:: python
        :linenos:

        from alaric import AQ
        from alaric.logical import NOT
        from alaric.comparison import EQ

        query = AQ(NOT(EQ("gamer_tag", "Skelmis")))
    """

    def __init__(
        self,
        *comparisons: Union[
            Union[ComparisonT, LogicalT, Buildable],
            List[Union[ComparisonT, LogicalT, Buildable]],
        ],
    ):
        self.comparisons: List[Union[ComparisonT, LogicalT]] = list(comparisons)

    def __repr__(self):
        return f"NOT(comparisons={self.comparisons})"

    def build(self) -> Dict[str, List[Dict]]:
        """Return this instance as a usable Mongo filter."""
        comparisons: List[Dict] = [c.build() for c in self.comparisons]
        return {"$not": comparisons}



from typing import Union

from bson import ObjectId as OID


class ObjectId(OID):
    """A thin wrapper over bson.ObjectId for usage within Alaric"""

    def __init__(self, object_id: Union[str, "ObjectId", bytes]):
        super().__init__(object_id)

    @property
    def object_id(self) -> Union[str, "ObjectId", bytes]:
        return self.__id



from typing import Dict, Union


class Exists:
    """
    Returns all documents that contain this field.

    This class can be used in conjunction with :py:class:`~alaric.meta.NEGATE`

    Parameters
    ----------
    field: str
        The field to check in.


    Lets match all documents where the field ``prefix`` exists

    .. code-block:: python

        from alaric import AQ
        from alaric.comparison import EXISTS

        query = AQ(EXISTS("prefix"))
    """

    def __init__(self, field: str):
        self.field: str = field
        self._val: bool = True

    def __repr__(self):
        return f"EXISTS(field='{self.field}')"

    def build(self) -> Dict[str, Dict[str, Union[int, str, float, bytes]]]:
        """Return this instance as a usable Mongo filter."""
        return {self.field: {"$exists": self._val}}



import logging
from collections import namedtuple

from .advanced_query import AQ
from .document import Document

__all__ = ("AQ", "Document")
__version__ = "1.1.4"
VersionInfo = namedtuple("VersionInfo", "major minor micro releaselevel serial")
version_info = VersionInfo(major=1, minor=1, micro=4, releaselevel="final", serial=0)
logging.getLogger(__name__).addHandler(logging.NullHandler())



import asyncio
import os
import secrets

from motor.motor_asyncio import AsyncIOMotorClient

from alaric import *
from alaric.comparison import *
from alaric.logical import *
from alaric.meta import *
from alaric.projections import *
from alaric.types import *


async def main():
    client = AsyncIOMotorClient(os.environ["MONGO"])
    db = client["COMPX323"]
    document: Document = Document(db, "movies")

    # r_1 = await document.find(EQ("_id", ObjectId("6297cf0186d144fe9b619135")))
    # print(r_1)
    print(AQ(IN("state", ["approved", "rejected"])).build())


if __name__ == "__main__":
    asyncio.run(main())



from typing import Dict, Union


class GT:
    """
    Asserts the provided field is greater to the provided value.

    This class can be used in conjunction with :py:class:`~alaric.meta.NEGATE`

    Parameters
    ----------
    field: str
        The field to check in.
    value: Union[int, str, float, bytes, dict]
        The value the field should be greater than.


    Lets match all documents where the field ``counter``
    is greater then 5.

    .. code-block:: python

        from alaric import AQ
        from alaric.comparison import GT

        query = AQ(GT("counter", 5))
    """

    def __init__(self, field: str, value: Union[int, str, float, bytes]):
        self.field: str = field
        assert not isinstance(value, (list, tuple, set))
        self.value: Union[int, str, float, bytes] = value

    def __repr__(self):
        return f"GT(field='{self.field}', value={self.value})"

    def build(self) -> Dict[str, Dict[str, Union[int, str, float, bytes]]]:
        """Return this instance as a usable Mongo filter."""
        return {self.field: {"$gt": self.value}}



from __future__ import annotations

from typing import TYPE_CHECKING, List, Dict, Union

if TYPE_CHECKING:
    from alaric.abc import ComparisonT, LogicalT, Buildable


class AND:
    """Conduct a logical AND between all items passed to the constructor.


    Lets build a query which returns all results where the field ``discord``
    is equal to ``Skelmis#9135`` and ``gamer_tag`` is equal to ``Skelmis``

    .. code-block:: python
        :linenos:

        from alaric import AQ
        from alaric.logical import AND
        from alaric.comparison import EQ

        query = AQ(AND(EQ("discord", "Skelmis#9135"), EQ("gamer_tag", "Skelmis")))
    """

    def __init__(
        self,
        *comparisons: Union[
            Union[ComparisonT, LogicalT, Buildable],
            List[Union[ComparisonT, LogicalT, Buildable]],
        ],
    ):
        self.comparisons: List[Union[ComparisonT, LogicalT, Buildable]] = list(
            comparisons
        )

    def __repr__(self):
        return f"AND(comparisons={self.comparisons})"

    def build(self) -> Dict[str, List[Dict]]:
        """Return this instance as a usable Mongo filter."""
        comparisons: List[Dict] = [c.build() for c in self.comparisons]
        return {"$and": comparisons}



from typing import Dict, Union

from alaric.types import ObjectId


class EQ:
    """
    Asserts the provided field is equal to the provided value.

    Parameters
    ----------
    field: str
        The field to check in.
    value: Union[int, str, float, bytes, dict, ObjectId]
        The value the field should equal.

    Notes
    -----
    This also works on matching items in arrays
    in an OR based matching approach.


    Lets match a document with an ``_id`` equal to ``1``

    .. code-block:: python

        from alaric import AQ
        from alaric.comparison import EQ

        query = AQ(EQ("_id", 1))
    """

    def __init__(
        self, field: str, value: Union[int, str, float, bytes, dict, ObjectId]
    ):
        self.field: str = field
        assert not isinstance(value, (list, tuple, set))
        self.value: Union[int, str, float, bytes, dict, ObjectId] = value
        self._operator = "$eq"

    def __repr__(self):
        return f"EQ(field='{self.field}', value={self.value})"

    def build(
        self,
    ) -> Dict[str, Dict[str, Union[int, str, float, bytes, dict, ObjectId]]]:
        """Return this instance as a usable Mongo filter."""
        return {self.field: {self._operator: self.value}}



from typing import runtime_checkable, Protocol, Dict


@runtime_checkable
class Filterable(Protocol):
    """Protocol for class based Queries."""

    def as_filter(self) -> Dict:
        """Returns a dictionary that represents
        a filter required to return this object.
        """
        ...



from .comparison_eq import EQ
from .comparison_in import IN
from .comparison_exists import Exists
from .comparison_less_than import LT
from .comparison_greater_than import GT

EXISTS = Exists
__all__ = ("EQ", "IN", "Exists", "GT", "LT", "EXISTS")



from typing import Any, Dict, List


class Hide:
    """Hide this field in the returned data."""

    def __init__(self, *fields: Any):
        self.fields: List[Any] = list(fields)

    def __repr__(self):
        return f"Hide({self.fields})"

    def build(self) -> Dict:
        return {field: 0 for field in self.fields}
